@@ -1,6 +1,7 @@
 package TAPx::Parser::Aggregator;
 
 use strict;
+use warnings;
 use vars qw($VERSION);
 
 =head1 NAME
@@ -9,11 +10,11 @@ TAPx::Parser::Aggregator - Aggregate TAPx::Parser results.
 
 =head1 VERSION
 
-Version 0.50_07
+Version 0.12
 
 =cut
 
-$VERSION = '0.50_07';
+$VERSION = '0.12';
 
 =head1 SYNOPSIS
 
@@ -31,7 +32,7 @@ $VERSION = '0.50_07';
     printf $summary, 
            scalar $aggregate->passed, 
            scalar $aggregate->failed,
-           scalar $aggregate->todo_passed;
+           scalar $aggregate->todo_failed;
 
 =head1 DESCRIPTION
 
@@ -59,10 +60,8 @@ BEGIN {
       passed
       skipped
       todo
-      todo_passed
+      todo_failed
       total
-      wait
-      exit
     );
     $SUMMARY_METHOD_FOR{total} = 'tests_run';
 
@@ -112,22 +111,26 @@ Trying to reuse a description is a fatal error.
 
 =cut
 
-sub add {
-    my ( $self, $description, $parser ) = @_;
-    if ( exists $self->{parser_for}{$description} ) {
-        $self->_croak("You already have a parser for ($description)");
-    }
-    push @{ $self->{parse_order} } => $description;
-    $self->{parser_for}{$description} = $parser;
+{
 
-    while ( my ( $summary, $method ) = each %SUMMARY_METHOD_FOR ) {
-        if ( my $count = $parser->$method ) {
-            $self->{$summary} += $count;
-            push @{ $self->{"descriptions_for_$summary"} } => $description;
+    sub add {
+        my ( $self, $description, $parser ) = @_;
+        if ( exists $self->{parser_for}{$description} ) {
+            $self->_croak("You already have a parser for ($description)");
         }
-    }
+        push @{ $self->{parse_order} } => $description;
+        $self->{parser_for}{$description} = $parser;
 
-    return $self;
+        while ( my ( $summary, $method ) = each %SUMMARY_METHOD_FOR ) {
+            if ( my $count = $parser->$method ) {
+                $self->{$summary} += $count;
+                push @{ $self->{"descriptions_for_$summary"} } =>
+                  $description;
+            }
+        }
+
+        return $self;
+    }
 }
 
 ##############################################################################
@@ -139,8 +142,8 @@ sub add {
   my @parsers = $aggregate->parsers(@descriptions);
 
 In scalar context without arguments, this method returns the number of parsers
-aggregated.  In list context without arguments, returns the parsers in the
-order they were added.
+aggregated.  In list context without arguments, returns the parsers
+themselves, in the order they were added.
 
 If arguments are used, these should be a list of descriptions used with the
 C<add> method.  Returns an array in list context or an array reference in
@@ -194,23 +197,15 @@ for an explanation of description.
 
 =item * todo
 
-=item * todo_passed
-
-=item * wait
-
-=item * exit
+=item * todo_failed
 
 =back
 
 For example, to find out how many tests unexpectedly succeeded (TODO tests
 which passed when they shouldn't):
 
- my $count        = $aggregate->todo_passed;
- my @descriptions = $aggregate->todo_passed;
-
-Note that C<wait> and C<exit> are the totals of the wait and exit
-statuses of each of the tests. These values are totalled only to provide
-a true value if any of them are non-zero.
+ my $count        = $aggregate->todo_failed;
+ my @descriptions = $aggregate->todo_failed;
 
 =cut
 
@@ -226,45 +221,6 @@ Returns the total number of tests run.
 
 sub total { shift->{total} }
 
-##############################################################################
-
-=head3 C<has_problems>
-
-  if ( $parser->has_problems ) {
-      ...
-  }
-
-This is a 'catch-all' method which returns true if any tests have currently
-failed, any TODO tests unexpectedly succeeded, or any parse errors.
-
-=cut
-
-sub has_problems {
-    my $self = shift;
-    return $self->failed
-      || $self->todo_passed
-      || $self->parse_errors
-      || $self->exit
-      || $self->wait;
-}
-
-##############################################################################
-
-=head3 C<todo_failed>
-
-  # deprecated in favor of 'todo_passed'.  This method was horribly misnamed.
-
-This was a badly misnamed method.  It indicates which TODO tests unexpectedly
-succeeded.  Will now issue a warning and call C<todo_passed>.
-
-=cut
-
-sub todo_failed {
-    warn
-      '"todo_failed" is deprecated.  Please use "todo_passed".  See the docs.';
-    goto &todo_passed;
-}
-
 sub _croak {
     my $proto = shift;
     require Carp;
